@@ -11,16 +11,20 @@ from threading import RLock as TRLock
 from typing import Any, Dict, List
 from urllib.parse import quote_plus, unquote_plus
 
-import multivolumefile
-import py7zr
+try:
+    import multivolumefile
+    import py7zr
+except ImportError:
+    multivolumefile = None
+    py7zr = None
 from tqdm.auto import tqdm
 
 from todus3 import __app_name__, __version__
 from todus3.client import ToDusClient
-from todus3.util import normalize_phone_number
+from todus3.util import ErrorCode, normalize_phone_number, tqdm_logging
 
 formatter = logging.Formatter("%(levelname)s-%(name)s-%(asctime)s-%(message)s")
-logger = logging.getLogger(__name__)
+logger = logging.getLogger(__app_name__)
 logger.setLevel(logging.INFO)
 handler_default = logging.StreamHandler()
 handler_default.setFormatter(formatter)
@@ -33,7 +37,7 @@ logger.addHandler(handler_error)
 client = ToDusClient()
 config = ConfigParser()
 MAX_RETRY = 3
-DOWN_TIMEOUT = 60 * 20  # 20 MINS
+DOWN_TIMEOUT = 30  # seconds
 MAX_WORKERS = 3
 PY3_7 = sys.version_info[:2] >= (3, 7)
 
@@ -59,7 +63,7 @@ def split_upload(
     logger.info("Compressing parts ...")
 
     with TemporaryDirectory() as tempdir:
-        with multivolumefile.open(
+        with multivolumefile.open(  # type: ignore
             Path(f"{tempdir}/{filename}.7z"),
             "wb",
             volume=part_size,
@@ -146,7 +150,7 @@ def register(client: ToDusClient, phone: str) -> str:
     client.request_code(phone)
     pin = input("Enter PIN:").strip()
     password = client.validate_code(phone, pin)
-    logger.debug("PASSWORD: %s", password)
+    tqdm_logging(logging.DEBUG, f"PASSWORD: {password}")
     return password
 
 
@@ -174,6 +178,11 @@ def _upload(token: str, args: argparse.Namespace, max_retry: int):
         filename = Path(path).name
         logger.info(f"Uploading: {filename}")
         if args.part_size:
+            if py7zr is None:
+                raise ImportError(
+                    "ModuleNotFoundError: No module named 'py7zr'\n"
+                    f"Install extra dependency like `pip install {__app_name__}[7z]`"
+                )
             txt = split_upload(token, path, args.part_size, max_retry=max_retry)
             logger.info(f"TXT: {txt}")
         else:
@@ -213,9 +222,9 @@ def _download(
         if urls:
             count_files = len(urls)
             plural = "" if count_files <= 1 else "s"
-            logger.debug(f"Downloading: {count_files} file{plural}")
+            tqdm_logging(logging.DEBUG, f"Downloading: {count_files} file{plural}")
 
-        logger.debug(f"Downloading: {file_uri}")
+        tqdm_logging(logging.DEBUG, f"Downloading: {file_uri}")
         file_uri, name = file_uri.split("?name=", maxsplit=1)
         name = unquote_plus(name)
         download_tasks[name] = (
@@ -242,13 +251,25 @@ def _download(
             client.session.close()
 
 
-def main() -> None:
+def main(action: str = "login", phone_number: str = "", folder_conf: str = ".") -> int:
+    """Main entrypoint adapted for Notebooks"""
     global client, config, logger
 
-    parser = get_parser()
-    args = parser.parse_args()
-    folder: str = args.folder
-    phone: str = normalize_phone_number(args.number)
+    folder: str = folder_conf
+    phone: str = ""
+    parser = None
+    error_code = ErrorCode.SUCCESS
+
+    if not phone_number:
+        parser = get_parser()
+        args = parser.parse_args()
+        folder = args.folder
+        phone = normalize_phone_number(args.number)
+        command = args.command
+    else:
+        phone = normalize_phone_number(str(phone_number))
+        command = action
+
     password: str = get_default(str, "password", phone, folder, "")
     max_retry: int = get_default(int, "max_retry", phone, folder, str(MAX_RETRY))
     config["DEFAULT"]["max_retry"] = str(max_retry)
@@ -264,27 +285,31 @@ def main() -> None:
     else:
         logger.setLevel(logging.DEBUG)
 
-    if not password and args.command != "login":
+    if not password and args.command != "login":  # type: ignore
         print("ERROR: account not authenticated, login first.")
-        return
+        return ErrorCode.MAIN
 
-    if args.command == "upload":
+    if command == "upload":
         token = client.login(phone, password)
-        logger.debug(f"Token: '{token}'")
-        _upload(token, args, max_retry)
-    elif args.command == "download":
+        tqdm_logging(logging.DEBUG, f"Token: '{token}'")
+        _upload(token, args, max_retry)  # type: ignore
+    elif command == "download":
         token = client.login(phone, password)
-        max_workers: int = args.max_threads
-        logger.debug(f"Token: '{token}'")
-        _download(token, args, down_timeout, max_retry, max_workers)
-    elif args.command == "login":
+        max_workers: int = args.max_threads  # type: ignore
+        tqdm_logging(logging.DEBUG, f"Token: '{token}'")
+        _download(token, args, down_timeout, max_retry, max_workers)  # type: ignore
+    elif command == "login":
         password = register(client, phone)
         token = client.login(phone, password)
-        logger.debug(f"Token: '{token}'")
+        tqdm_logging(logging.DEBUG, f"Token: '{token}'")
 
         config["DEFAULT"]["password"] = password
         config["DEFAULT"]["token"] = token
-    else:
+    elif parser:
         parser.print_usage()
+    else:
+        raise RuntimeError("Argument Error")
 
+    error_code = client.error_code
     save_config(phone, folder)
+    return error_code
